//
//  SetGameView.swift
//  Squiggle
//
//  Created by Sameh Fakhouri on 3/22/23.
//

import SwiftUI

struct SetGameView: View {
    
    var setCards: [SetCard] = [
        SetCard(shape: SetCard.Shapes.squiggle,
                shade: SetCard.Shades.outlined,
                color: SetCard.Colors.green,
                count: 1,
                id: 1,
                isSelected: true,
                isMatched: false,
                isMismatched: false),

        SetCard(shape: SetCard.Shapes.squiggle,
                shade: SetCard.Shades.filled,
                color: SetCard.Colors.red,
                count: 2,
                id: 2,
                isSelected: false,
                isMatched: true,
                isMismatched: false),

        SetCard(shape: SetCard.Shapes.squiggle,
                shade: SetCard.Shades.striped,
                color: SetCard.Colors.purple,
                count: 3,
                id: 3,
                isSelected: false,
                isMatched: false,
                isMismatched: true),

        SetCard(shape: SetCard.Shapes.squiggle,
                shade: SetCard.Shades.striped,
                color: SetCard.Colors.green,
                count: 1,
                id: 4,
                isSelected: false,
                isMatched: false,
                isMismatched: false)
    ]
    
    var body: some View {
        VStack {
            gameBody
                .padding(.horizontal)
            Spacer()
            HStack {
                newGame
                Spacer()
                peek
                Spacer()
                plusThree
            }
            .padding(.horizontal)
        }
    }
    
    var gameBody: some View {
        AspectVGrid(items: setCards, aspectRatio: 3/2){ setCard in
            SetCardView(setCard)
                .padding(5)
        }
    }
    
    var newGame: some View {
        Button {
            // action
        } label: {
            VStack {
                Image(systemName: "arrowtriangle.right.circle")
                    .imageScale(.large)
                    .font(.largeTitle)
                Text("New Game")
                    .font(.caption)
            }
            
        }
    }
    
    var peek: some View {
        Button {
            // action
        } label: {
            VStack {
                Image(systemName: "eye")
                    .imageScale(.large)
                    .font(.largeTitle)
                Text("Peek")
                    .font(.caption)
            }
            
        }
    }
    
    var plusThree: some View {
        Button {
            // action
        } label: {
            VStack {
                Image(systemName: "plus.circle")
                    .imageScale(.large)
                    .font(.largeTitle)
                Text("+3")
                    .font(.caption)
            }
            
        }
    }
}


struct SetCardView: View {
    private let setCard: SetCard
    private let lineColor: Color
    
    init(_ setCard: SetCard) {
        self.setCard = setCard
        if setCard.isSelected {
            lineColor = Color.black
        } else if setCard.isMatched {
            lineColor = Color.green
        } else if setCard.isMisMatched {
            lineColor = Color.red
        } else {
            lineColor = Color.gray
        }
    }

    var body: some View {
        GeometryReader { geometry in
            ZStack {
                let roundedRectangle = RoundedRectangle(cornerRadius: 10)
                roundedRectangle.fill()
                    .foregroundColor(Color.white)
                roundedRectangle.stroke(lineWidth: 3.0)
                    .foregroundColor(lineColor)
                if setCard.shade == .outlined {
                    getPath(for: setCard, in: geometry.frame(in: .local))
                        .stroke(colorForPath(for: setCard))
                } else if setCard.shade == .filled {
                    getPath(for: setCard, in: geometry.frame(in: .local))
                        .fill(colorForPath(for: setCard))
                } else if setCard.shade == .striped {
                    getPath(for: setCard, in: geometry.frame(in: .local))
                        .stroke(colorForPath(for: setCard))
                        .clipShape(getPath(for: setCard, in: geometry.frame(in: .local)))
                }
            }
        }
        
    }
    
    
    private func getPath(for setCard: SetCard, in rect: CGRect) -> Path {
        var path: Path
        switch setCard.shape {
        case .diamond:
            path = Diamond().path(in: rect)
        case .oval:
            path = Oval().path(in: rect)
        case .squiggle:
            path = Squiggle().path(in: rect)
        }
        
        path = replicatePath(path, for: setCard, in: rect)
        
        if setCard.shade == .striped {
            path.addPath(getStripedPath(in: rect))
        }
        
        return path
    }

    private func colorForPath(for setCard: SetCard) -> Color {
        switch setCard.color {
        case .green:
            return Color.green
        case .red:
            return Color.red
        case .purple:
            return Color.purple
        }
    }
    
    private func replicatePath(_ path: Path, for setCard: SetCard, in rect: CGRect) -> Path {
        var leftTwoPathTranslation: CGPoint {
            return CGPoint(
                x: rect.width * -0.15,
                y:0.0)
        }

        var rightTwoPathTranslation: CGPoint {
            return CGPoint(
                x: rect.width * 0.15,
                y:0.0)
        }

        var leftThreePathTranslation: CGPoint {
            return CGPoint(
                x: rect.width * -0.25,
                y:0.0)
        }

        var rightThreePathTranslation: CGPoint {
            return CGPoint(
                x: rect.width * 0.25,
                y:0.0)
        }
        
        var replicatedPath = Path()
        
        if (setCard.count == 1) {
            replicatedPath = path
        } else if (setCard.count == 2) {
            let leftTransform = CGAffineTransform(translationX: leftTwoPathTranslation.x,
                                                  y: leftTwoPathTranslation.y)
            let rightTransform = CGAffineTransform(translationX: rightTwoPathTranslation.x,
                                                   y: rightTwoPathTranslation.y)
            
            replicatedPath.addPath(path, transform: leftTransform)
            replicatedPath.addPath(path, transform: rightTransform)
        } else {
            let leftTransform = CGAffineTransform(translationX: leftThreePathTranslation.x,
                                                  y: leftThreePathTranslation.y)
            let rightTransform = CGAffineTransform(translationX: rightThreePathTranslation.x,
                                                   y: rightThreePathTranslation.y)

            replicatedPath.addPath(path, transform: leftTransform)
            replicatedPath.addPath(path, transform: rightTransform)
            replicatedPath.addPath(path)
        }
        
        return replicatedPath
    }

    private func getStripedPath(in rect: CGRect) -> Path {
        var stripedPath = Path()
        
        let dy: CGFloat = rect.height / 20.0
        var start = CGPoint(x: 0.0, y: dy)
        var end = CGPoint(x: rect.width, y: dy)
        while start.y < rect.height {
            stripedPath.move(to: start)
            stripedPath.addLine(to: end)
            start.y += dy
            end.y += dy
        }
        
        return stripedPath
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        SetGameView()
    }
}
